//Created by ZLv (Евгений Лежнин) <z_lezhnin@mail2000.ru> (thanks Boris Churzin<boris.churzin@gmail.com>, Nikita Krupen'ko and Nikhil Marathe)
//tested in amarok 2.4
Importer.loadQtBinding("qt.core");
Importer.loadQtBinding("qt.gui");
Importer.loadQtBinding("qt.uitools");
//load translations
const lang = QLocale.system().name().substring(0,2);
var translator = new QTranslator();
translator.load("locale."+lang,Amarok.Info.scriptPath()+"/translations/");
QCoreApplication.installTranslator(translator);
function qsTr(str){
    return QCoreApplication.translate("Moodbar_main",str);
}
//check moodbar
if (!QProcess.startDetached("moodbar",["2>/dev/null",">/dev/null"])) { Amarok.alert(qsTr("Can't find moodbar!\nPlease, follow instructions from: http://amarok.kde.org/wiki/Moodbar")); Amarok.end(); }
var processNumMax;
var processNumMaxForCurrent;
var currentTracksCount;
var processNum;
//default 3(8 before);
processNumMax = 3;
processNumMax = Amarok.Script.readConfig("moodbar_processNumMax", processNumMax+"");
processNumMaxForCurrent = processNumMax*2;
currentTracksCount = 3;
processNum = 0;
var listNotCreated = new Array();
var listNotCreatedFirstDefault=50;
var listNotCreatedFirst=listNotCreatedFirstDefault;
var listNotCreatedLast= listNotCreatedFirstDefault-1;
//////////////////////
///////////////////
// functions for signals
///////////////////
Amarok.Playlist.trackInserted.connect(function(frow, row)
{
    //get filenames to see what files inserted
    var filenames = Amarok.Playlist.filenames();
    var i;
    for (i=frow; i<=row; i++)
    {
        var path = filenames[i];
        create_mood(path, processNumMax);
    }
});

//create mood for playing file
Amarok.Engine.trackChanged.connect(function() {
    generate_prioritied(Amarok.Playlist.activeIndex());
});
//create moods near playing file
function generate_prioritied(track)
{
    var filenames = Amarok.Playlist.filenames();
    var i;
    for (i=track; i<track+currentTracksCount; i++)
    {
        var path = filenames[i];
        create_mood_prioritied(path,processNumMaxForCurrent);
    }
}
function process_ended()
{
    processNum--;
    Amarok.debug(qsTr("processes : ") + processNum);
}
///////////////////
// work functions
///////////////////
function create_one_mood(mood, path, test)
{
    test = test || false;
    if (test)
    {
        if (create_mood_filename(path)==null) return;
    }
    var process = new QProcess(null);
    var args = ["-o", mood, path];
    process.start("moodbar",args);
    process["finished(int,QProcess::ExitStatus)"].connect(process_ended);
    processNum++;
    Amarok.debug(qsTr("processes : ") + processNum + qsTr(" with mood: ") + mood);
}
function create_mood_filename(path)
{
    var mood = get_mood_path(path);
    if (mood==null) return null;
    var file = new QFile(mood);
    if (file.exists())
    {
        if (file.size()==0)
        {
            if (!file.remove()) return null;
        }
        else return null;
    }
    return mood;
}
function create_mood(path, max)
{
    var mood = create_mood_filename(path);
    if (mood==null) return null;
    if (processNum>=max)
    {
        listNotCreatedLast++;
        listNotCreated[listNotCreatedLast] = new Object();
        listNotCreated[listNotCreatedLast].mood = mood;
        listNotCreated[listNotCreatedLast].path = path;
        listNotCreated[listNotCreatedLast].max = max;
    }
    else create_one_mood(mood,path);
}
//create prioritied mood
function create_mood_prioritied(path, max)
{
    var mood = create_mood_filename(path);
    if (mood==null) return null;
    if (processNum>=max)
    {
        if (listNotCreatedFirst>0)
        {
            listNotCreatedFirst--;
            listNotCreated[listNotCreatedFirst] = new Object();
            listNotCreated[listNotCreatedFirst].mood = mood;
            listNotCreated[listNotCreatedFirst].path = path;
            listNotCreated[listNotCreatedFirst].max = max;
        }
        else
        {
            listNotCreatedLast++;
            listNotCreated[listNotCreatedLast] = new Object();
            listNotCreated[listNotCreatedLast].mood = mood;
            listNotCreated[listNotCreatedLast].path = path;
            listNotCreated[listNotCreatedLast].max = max;
        }
    }
    else create_one_mood(mood,path);
}
//should be replaced with QUrl.isLocalFile() in the future
function isLocalFile(path)
{
    if (path=="" || path.isEmpty()) return 1;
    Amarok.debug("==================================");
    Amarok.debug(path);
    var url = new QUrl(path);
    if (url.host()!="" || url.scheme()!="" || url.path()!=path)
    {
        return false;
    }
    return true;
}

function get_mood_path(path)
{
    //should be replaced with QUrl.isLocalFile() in the future
    if (path=="" || !isLocalFile(path))
    {
        return null;
    }
    var info = new QFileInfo(path);
    var mood = info.path()+"/."+info.completeBaseName()+".mood";
    
    var infodir = new QFileInfo(info.path());
    if (!infodir.isWritable() || info.suffix()=="")
    {
        delete info;
        delete infodir;
        return null;
    }
    
    delete info;
    delete infodir;
    return mood;
}

function generate_moods()
{
    var filenames = Amarok.Playlist.filenames();
    var i;
    for (i=0;i<filenames.length; i++)
    {
        create_mood(filenames[i], processNumMax);
    }
}

///////////////////////
//GUI
///////////////////////
OptionsDialog = function(parent)
{
    this.widget = new QWidget(parent);
    this.widget.windowTitle = qsTr("Configuration - Moodbar Generator");
    
    this.edit = new QLineEdit(processNumMax+"");
    this.label = new QLabel(qsTr("How many tracks generate in the same time"));
    this.button = new QPushButton(qsTr("Ok"));
    
    this.layout = new QVBoxLayout(this.widget);
    this.layout.addWidget(this.label, 0,1);
    this.layout.addWidget(this.edit,  0,1);
    this.layout.addWidget(this.button,0,1);
    
    var self = this;
    
    this.button['clicked()'].connect(function ()
    {
        var num = self.edit.text;
        //some check
        if ((num-1)+1==num && num>0)
        {
            processNumMax = num;
            processNumMaxForCurrent = num*2;
            Amarok.Script.writeConfig("moodbar_processNumMax", processNumMax+"");
            self.widget.hide();
        }
    });
};

OptionsDialog.prototype.show = function() {
    this.widget.show();
};

OptionsDialog.prototype.hide = function() {
    this.widget.hide();
};
///////////////////////////
options = new OptionsDialog(this);

Amarok.Window.addSettingsMenu("moodbarSettings",qsTr("Configure Moodbar Script"),"configure");

Amarok.Window.SettingsMenu.moodbarSettings["triggered()"].connect(function() 
{ options.show(); });

//generate moods on startup
generate_moods();

var interval=100;
//timer function, that should create not created moods
var qo = new QObject();
qo.event = function(qevent) {
    if (listNotCreatedLast>=listNotCreatedFirst)
    {
        if (processNum<listNotCreated[listNotCreatedFirst].max)
        {
            Amarok.debug(qsTr("timer, left: ") + (listNotCreatedLast-listNotCreatedFirst));
            create_one_mood(listNotCreated[listNotCreatedFirst].mood,listNotCreated[listNotCreatedFirst].path,true);
            listNotCreatedFirst++;
        }
    }
    else
    {
        listNotCreatedFirst=listNotCreatedFirstDefault;
        listNotCreatedLast= listNotCreatedFirstDefault-1;
    }
}
qo.timerID = qo.startTimer(interval);
